package com.gigachatwoot.connectors.chatwoot.client;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.gigachatwoot.connectors.chatwoot.services.BotDefinitionService;
import com.gigachatwoot.connectors.chatwoot.dto.CSMLMessageDTO;
import com.gigachatwoot.connectors.chatwoot.dto.CSMLRequestDTO;
import com.gigachatwoot.connectors.chatwoot.dto.CSMLResponseDTO;
import com.gigachatwoot.connectors.chatwoot.models.ChatMessage;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.*;
import org.springframework.stereotype.Component;
import org.springframework.web.client.HttpServerErrorException;
import org.springframework.web.client.RestTemplate;

import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@Slf4j
@Getter
@Setter
@Component
@RequiredArgsConstructor
public class CSMLClient {
    private final RestTemplate restTemplate;
    private final BotDefinitionService botDefinitionService;

    public void sendMessageHistoryToCSML(Long accountId, Long conversationId, List<ChatMessage> sortedMessages,
                                         String csmlUrl) {
        String url = csmlUrl + "/run";  // Update according to the appropriate endpoint

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);

        List<CSMLMessageDTO> history = sortedMessages.stream()
            .map(msg -> {
                String id = msg.getId().toString();
                String senderType = (msg.getSender() != null) ? msg.getSender().getType() : "user";
                String content = (msg.getContent() != null) ? msg.getContent() : "";
                String messageType = msg.getMessageType() == 0 ? "incoming" : "outgoing";

                return new CSMLMessageDTO(
                    id,
                    senderType,
                    content,
                    messageType
                );
            })
            .collect(Collectors.toList());

        CSMLMessageDTO lastMessage = history.get(history.size() - 1);

        CSMLRequestDTO.Bot bot = botDefinitionService.getBot();

        CSMLRequestDTO.Event.Client client = new CSMLRequestDTO.Event.Client(
            "llmbot_id",
            UUID.randomUUID().toString(),
            UUID.randomUUID().toString()
        );

        CSMLRequestDTO.Event.Payload.Content content = new CSMLRequestDTO.Event.Payload.Content(
            lastMessage.getContent(),
            accountId,
            conversationId,
            history
        );

        CSMLRequestDTO.Event.Payload payload = new CSMLRequestDTO.Event.Payload(
            "text",
            content,
            false
        );

        CSMLRequestDTO.Event.Metadata metadata = new CSMLRequestDTO.Event.Metadata(
            "username",
            "email@email.com"
        );

        CSMLRequestDTO.Event event = new CSMLRequestDTO.Event(
            UUID.randomUUID().toString(),
            client,
            payload,
            metadata,
            30,
            false
        );

        CSMLRequestDTO requestBody = new CSMLRequestDTO(bot, event);

        HttpEntity<CSMLRequestDTO> requestEntity = new HttpEntity<>(requestBody, headers);
        try {
            log.info("sendMessageHistoryToCSML: Sending request body: " + new ObjectMapper().writeValueAsString(requestBody));
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }

        ResponseEntity<CSMLResponseDTO> response;
        try {
            response = restTemplate.exchange(
                url,
                HttpMethod.POST,
                requestEntity,
                CSMLResponseDTO.class
            );
        } catch (HttpServerErrorException e) {
            log.error("sendMessageHistoryToCSML: Server error: " + e.getMessage());
            log.error("sendMessageHistoryToCSML: Response body: " + e.getResponseBodyAsString());
            throw e;
        }

        if (response.getStatusCode() != HttpStatus.OK) {
            log.error("sendMessageHistoryToCSML: Failed to process message with CSML: {}", response.getStatusCode());
        }
    }
}
