package com.gigachatwoot.connectors.chatwoot.services;

import com.gigachatwoot.connectors.chatwoot.dto.CSMLRequestDTO;
import com.gigachatwoot.connectors.chatwoot.dto.CSMLFlowInfoDTO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.*;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.stream.Collectors;

@Slf4j
@Service
public class BotDefinitionService {

    @Value("${chatwoot.connector.url}")
    private String chatwootConnectorUrl;

    @Value("${gigachat.connector.url}")
    private String gigachatConnectorUrl;

    @Value("${neuralfilter.url}")
    private String neuralFilterUrl;

    @Value("${csml.api.base.url}")
    private String csmlUrl;

    private final ReentrantReadWriteLock lock = new ReentrantReadWriteLock();

    private final CSMLRequestDTO.Bot bot;

    private final RestTemplate restTemplate;

    public BotDefinitionService(RestTemplate restTemplate) {
        this.restTemplate = restTemplate;
        this.bot = new CSMLRequestDTO.Bot(
            "llmbot_id",
            "LLMBot",
            "LegalQuestion",
            List.of(
                new CSMLRequestDTO.Bot.Flow(
                    UUID.randomUUID().toString(),
                    "AssigneeRequest",
                    "start: do assignee_request = HTTP(\"" + chatwootConnectorUrl + "/csml/response\").post({\"account_id\": \"{{event.account_id}}\", \"conversation_id\": \"{{event.conversation_id}}\", \"message\": \"Поняли, подождите оператора.\", \"needs_operator\": true}).send() goto end",
                    List.of()
                ),
                new CSMLRequestDTO.Bot.Flow(
                    UUID.randomUUID().toString(),
                    "CSMLService",
                    "fn checkTriggerWords(arrayWords) { \n foreach (word) in arrayWords { \n do hasTriggerWord = Find(word, \"{{event.text}}\") \n if (hasTriggerWord) { return true } \n } \n return false \n } \n fn checkAssigneeCall(assigneeCall) { \n do isAssigneeCalled = Find(assigneeCall, \"{{event.text}}\") \n if (isAssigneeCalled) { return true } \n return false }",
                    List.of()
                ),
                new CSMLRequestDTO.Bot.Flow(
                    UUID.randomUUID().toString(),
                    "ClassifierRequest",
                    "start: \n do pfReq = HTTP(\"" + neuralFilterUrl + "/process\").post({\"message\": \"{{event.text}}\"}).send() \n if (pfReq.response.match(\"Юридический вопрос\")) \n goto flow LegalQuestion \n else if (pfReq.response.match(\"Общий вопрос\")) \n goto flow GeneralQuestion",
                    List.of()
                ),
                new CSMLRequestDTO.Bot.Flow(
                    UUID.randomUUID().toString(),
                    "GeneralQuestion",
                    "start: \n goto flow AssigneeRequest",
                    List.of()
                ),
                new CSMLRequestDTO.Bot.Flow(
                    UUID.randomUUID().toString(),
                    "LegalQuestion",
                    "start: \n do llmResponse = HTTP(\"" + gigachatConnectorUrl + "/api/v1/get-llm-answer\").post(\"{{event.llm_request}}\").send() \n do chatwootReq = HTTP(\"" + chatwootConnectorUrl + "/csml/response\").post({\"account_id\": \"{{event.account_id}}\", \"conversation_id\": \"{{event.conversation_id}}\", \"message\": \"{{llmResponse}}\", \"needs_operator\": false}).send() \n goto end",
                    List.of()
                ),
                new CSMLRequestDTO.Bot.Flow(
                    UUID.randomUUID().toString(),
                    "SortingLayer",
                    "import checkTriggerWords from CSMLService \n import checkAssigneeCall from CSMLService \n start: \n do arrayTriggerWords = [\"оператор\"] \n do assigneeCall = \"/operator\" \n do isAssigneeProbablyRequested = checkTriggerWords(arrayTriggerWords) \n if (isAssigneeProbablyRequested) { \n say \"Возможно, вы хотите обсудить вопрос с оператором. \n Чтобы это сделать, введите /operator.\" \n hold \n do isAssigneeRequested = checkAssigneeCall(assigneeCall) \n if (isAssigneeRequested) { \n goto flow AssigneeRequest \n } \n } \n else goto flow ClassifierRequest",
                    List.of()
                )
            )
        );
    }

    public void updateBotFlows(CSMLFlowInfoDTO updateRequest) {
        lock.writeLock().lock();
        try {
            validateDefaultFlow(updateRequest.getDefaultFlow(), updateRequest.getFlows());

            validateUniqueFlowNames(updateRequest.getFlows());

            List<CSMLRequestDTO.Bot.Flow> updatedFlows = updateRequest.getFlows().stream()
                .map(flowDTO -> new CSMLRequestDTO.Bot.Flow(
                    UUID.randomUUID().toString(),
                    flowDTO.getName(),
                    flowDTO.getContent(),
                    Collections.emptyList()
                ))
                .collect(Collectors.toList());

            bot.setDefaultFlow(updateRequest.getDefaultFlow());
            bot.setFlows(updatedFlows);

            // Валидация на движке CSML
            validateCSML();

            log.info("Bot flows updated successfully. Default flow set to '{}'.", updateRequest.getDefaultFlow());
        } finally {
            lock.writeLock().unlock();
        }
    }

    private void validateDefaultFlow(String defaultFlow, List<CSMLFlowInfoDTO.FlowDTO> flows) {
        boolean flowExists = flows.stream()
            .anyMatch(flow -> flow.getName().equals(defaultFlow));

        if (!flowExists) {
            throw new IllegalArgumentException("Default flow must match the name of one of the flows.");
        }
    }

    private void validateUniqueFlowNames(List<CSMLFlowInfoDTO.FlowDTO> flows) {
        Set<String> flowNames = new HashSet<>();
        for (CSMLFlowInfoDTO.FlowDTO flow : flows) {
            if (!flowNames.add(flow.getName())) {
                throw new IllegalArgumentException("Flow names must be unique. Duplicate name found: " + flow.getName());
            }
        }
    }

    private void validateCSML() {
        String url = csmlUrl + "/validate";

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);

        HttpEntity<CSMLRequestDTO.Bot> requestEntity = new HttpEntity<>(bot, headers);

        ResponseEntity<Map> response = restTemplate.exchange(
            url,
            HttpMethod.POST,
            requestEntity,
            Map.class
        );

        if (response.getStatusCode() != HttpStatus.OK || response.getBody() == null) {
            throw new IllegalStateException("Failed to validate CSML flows on the engine.");
        }

        Boolean isValid = (Boolean) response.getBody().get("valid");
        if (!isValid) {
            List<Map<String, Object>> errors = (List<Map<String, Object>>) response.getBody().get("errors");
            throw new IllegalStateException("CSML validation failed with errors: " + errors);
        }
    }

    public CSMLRequestDTO.Bot getBot() {
        lock.readLock().lock();
        try {
            return bot;
        } finally {
            lock.readLock().unlock();
        }
    }

    public CSMLFlowInfoDTO getFlowInfo() {
        lock.readLock().lock();
        try {
            List<CSMLFlowInfoDTO.FlowDTO> flowDTOs = bot.getFlows().stream()
                .map(flow -> new CSMLFlowInfoDTO.FlowDTO(
                    flow.getName(),
                    flow.getContent()
                ))
                .collect(Collectors.toList());

            return new CSMLFlowInfoDTO(bot.getDefaultFlow(), flowDTOs);
        } finally {
            lock.readLock().unlock();
        }
    }
}
